@@ -246,6 +246,7 @@ static int kerberos_verify_password(sasl_conn_t *conn,
     char instance[INST_SZ];
     AUTH_DAT kdata;
     char *srvtab = "";
+    char *user_inst = "";
     sasl_getopt_t *getopt;
     void *context;
 
@@ -258,8 +259,10 @@ static int kerberos_verify_password(sasl_conn_t *conn,
     if (_sasl_getcallback(conn, SASL_CB_GETOPT, &getopt, &context) 
 	== SASL_OK) {
 	getopt(context, NULL, "srvtab", (const char **) &srvtab, NULL);
+	getopt(context, NULL, "plain_inst", (const char **) &user_inst, NULL);
 
 	if (!srvtab) srvtab = "";
+	if (!user_inst) user_inst = "";
     }
 
 
@@ -273,7 +276,7 @@ static int kerberos_verify_password(sasl_conn_t *conn,
     /* First try Kerberos string-to-key */
     des_string_to_key((char *) passwd, &key);
     
-    result = krb_get_in_tkt((char *)user, "", realm,
+    result = krb_get_in_tkt((char *)user, user_inst, realm,
 			    "krbtgt", realm, 1, use_key, NULL, &key);
 
     if (result == INTK_BADPW) {
@@ -283,7 +286,7 @@ static int kerberos_verify_password(sasl_conn_t *conn,
 	lcase(cell);
 	krb_afs_string_to_key(passwd, key, cell);
     
-	result = krb_get_in_tkt((char *)user, "", realm,
+	result = krb_get_in_tkt((char *)user, user_inst, realm,
 				"krbtgt", realm, 1, use_key, NULL, &key);
     }
 
@@ -366,6 +369,45 @@ static int shadow_verify_password(sasl_conn_t *conn __attribute__((unused)),
 }
 #endif /* HAVE_GETSPNAM */
 
+#ifdef HAVE_SIA
+#include <sia.h>
+#include <siad.h>
+#include <arpa/inet.h>
+
+static int sia_verify_password(sasl_conn_t *conn,
+				  const char *userid,
+				  const char *password,
+				  const char *service __attribute__((unused)),
+				  const char *user_realm 
+				               __attribute__((unused)), 
+				  const char **reply)
+{
+  char *host;
+  struct sockaddr_in *addr;
+  int argc = 1;
+  char *argv[2], *argv0 = "SASL";
+  argv[0] = argv0;
+  argv[1] = NULL;
+
+  if (!userid || !password) {
+      return SASL_BADPARAM;
+  }
+  if (reply) { *reply = NULL; }
+
+  /* Get the remote host */
+  if (sasl_getprop(conn, SASL_IP_REMOTE, (void **) &addr) != SASL_OK)
+      host = NULL;
+  else
+      host = inet_ntoa(*addr);
+
+  /* Try to validate */
+  if (sia_validate_user(NULL, argc, argv, host, userid, NULL, 0, NULL,
+      password) != SIASUCCESS)
+      return SASL_BADAUTH;
+  return SASL_OK;
+}
+#endif /* HAVE_SIA */
+
 #ifdef HAVE_GETPWNAM
 static int passwd_verify_password(sasl_conn_t *conn __attribute__((unused)),
 				  const char *userid,
@@ -863,6 +905,9 @@ struct sasl_verify_password_s _sasl_verify_password[] = {
 #ifdef HAVE_GETSPNAM
     { "shadow", &shadow_verify_password },
 #endif
+#ifdef HAVE_SIA
+    { "sia", &sia_verify_password },
+#endif
 #ifdef HAVE_GETPWNAM
     { "passwd", &passwd_verify_password },
 #endif
